/*
    QJpegRest - An interactive JPEG decoder, with restoration algorithms
    Copyright (C) 2007  Lluís Batlle i Rossell

    Please find the license in the provided LICENSE file.
*/
#include <Actions.h>
#include <ActionType.h>
#include <ActionCreator.h>
#include <ActionManager.h>

#include <FloatPlane.h>
#include <JPEGData.h>
#include <CoefsPlane.h>

#include <QCSKeepHF.h>

#include <QDialog>
#include <QSpinBox>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>

#include "QTConfigurable.h"
#include "QQCSKeepHF.moc"

QQCSKeepHF::QQCSKeepHF()
{
    GUIconfigurable = true;
    dialog = 0;
}

QQCSKeepHF::~QQCSKeepHF()
{
    /* This will free all the elements of the dialog */
    if (dialog)
        delete dialog;
}

void QQCSKeepHF::startQtConfiguration()
{
    if (!dialog)
    {
        dialog = new QDialog();
        dialog->setWindowTitle(trUtf8("Configurant") + QString(" QCSKeepHF"));
        QHBoxLayout *h = new QHBoxLayout(dialog);

        QVBoxLayout *vlabels = new QVBoxLayout();
        QLabel *l = new QLabel(trUtf8("Últims coeficients:"), dialog);
        l->setAlignment(Qt::AlignRight);

        QLabel *lvar = new QLabel(trUtf8("Nivell:"), dialog);
        lvar->setAlignment(Qt::AlignRight);
        vlabels->addWidget(l);
        vlabels->addWidget(lvar);
        h->addLayout(vlabels);

        QVBoxLayout *vspins = new QVBoxLayout();
        spinCoefs = new QSpinBox(dialog);
        spinCoefs->setMaximum(64);
        spinCoefs->setMinimum(0);
        spinCoefs->setSingleStep(1);
        vspins->addWidget(spinCoefs);
        spinLevel = new QDoubleSpinBox(dialog);
        spinLevel->setMaximum(10000);
        spinLevel->setMinimum(0);
        spinLevel->setSingleStep(1);
        vspins->addWidget(spinLevel);
        h->addLayout(vspins);

        QVBoxLayout *vbuttons = new QVBoxLayout();
        QPushButton *applyButton = new QPushButton(trUtf8("Aplicar"), dialog);
        QPushButton *resetButton = new QPushButton(trUtf8("Restablir"), dialog);

        vbuttons->addWidget(applyButton);
        vbuttons->addWidget(resetButton);
        h->addLayout(vbuttons);

        connect(applyButton, SIGNAL(released()), this, SLOT(setConfig()));
        connect(resetButton, SIGNAL(released()), this, SLOT(resetConfig()));
        resetConfig();
    }
    dialog->show();
}

void QQCSKeepHF::setConfig()
{
    HFlevel = spinLevel->value();
    latestCoefs = spinCoefs->value();
}

void QQCSKeepHF::resetConfig()
{
    spinLevel->setValue(HFlevel);
    spinCoefs->setValue(latestCoefs);
}

QQCSKeepHFCreator::QQCSKeepHFCreator()
{
    type = e_ImproveRawPlane;
}

void QQCSKeepHFCreator::init()
{
    ActionCreator *a = new QQCSKeepHFCreator();
    ActionManager::sreg("QCSKeepHF", a);
}

ImproveRawPlane * QQCSKeepHFCreator::createImproveRawPlane() const
{
    QQCSKeepHF *ptr;
    ptr = new QQCSKeepHF();
    return ptr;
}

bool QQCSKeepHFCreator::isapplicable(const JPEGParameters &p)
{
    return true;
}
