#include <stdio.h>
#include <pam.h>
#include <assert.h>
#include <string.h>
#include "libjpegrest.h"

void bitmap_pam_init(int *argn, char **argv)
{
	pnm_init(argn, argv);
}

int
bitmap_filename_pam_get(const char *filename, struct bitmap *bmap)
{
	FILE *input;
	int res;

	/* Opening the FILE */
	input = fopen(filename,"r");
	if (input == NULL)
	{
		fprintf(stderr,"The file \"%s\" cannot be opened.\n",
			filename);
		res = -1;
	} else
	{
		res = bitmap_stdio_pam_get(input, bmap);
		fclose(input);
	}
	
	return res;
}

int
bitmap_stdio_pam_get(FILE *input, struct bitmap *bmap)
{
	struct pam inpam;
	tuple **tuples;
	int row, column, plane;

	/* Reading the input files PAM header */
	tuples = pnm_readpam(input, &inpam, PAM_STRUCT_SIZE(tuple_type));
	if (tuples == NULL)
	{
		fprintf(stderr, "Mem malloc error.");
		return -1;
	}

	/* Get the number of planes */
	*bmap = bitmap_alloc(inpam.depth, inpam.width, inpam.height);

	for (row = 0; row < inpam.height; row++) {
		for (column = 0; column < inpam.width; ++column) {
			for (plane = 0; plane < bmap->nplanes; plane++)
				bmap->m[plane]->data[row *
					bmap->m[plane]->tda + column] =
					tuples[row][column][plane];
		}
	}

	/* Free the pnm memory */
	pnm_freepamarray(tuples, &inpam);
	return 0;
}

int
bitmap_filename_pam_put(const char *filename, struct bitmap *bmap)
{
	FILE *output;
	int res;

	/* Opening the FILE */
	output = fopen(filename,"w");
	if (output == NULL)
	{
		fprintf(stderr,"The file \"%s\" cannot be opened.\n",
			filename);
		res = -1;
	} else
	{
		res = bitmap_stdio_pam_put(output, bmap);
		fclose(output);
	}
	
	return res;
}

static unsigned char roundsample(float val)
{
    const float MAXVALUE = 255.;
    const float MINVALUE = 0.;

    val += 0.5;
    if (val > MAXVALUE)
        val = MAXVALUE;
    else if (val < MINVALUE)
        val = MINVALUE;

    return (unsigned char) val;
}


int
bitmap_stdio_pam_put(FILE *output, struct bitmap *bmap)
{
	struct pam outpam;
	tuple *tuplerow;
	int row, column, plane;

    assert(bmap->nplanes == 1); /* TODO: only implemented for PGM. */

    outpam.size = sizeof(outpam);
    outpam.len = outpam.size;
    outpam.file = output;
    outpam.format = PGM_FORMAT;
    outpam.plainformat = 0; /* false */
    outpam.width = bmap->width;
    outpam.height = bmap->height;
    outpam.depth = 1; /* Grayscale - 1 sample per tuple */
    outpam.allocation_depth = 0; /* Same as depth. */
    outpam.maxval = 255;
    strcpy(outpam.tuple_type, "GRAYSCALE"); /* PGM non transparent */

	pnm_writepaminit(&outpam);

    tuplerow = pnm_allocpamrow(&outpam);

	for (row = 0; row < bmap->height; row++) {
		for (column = 0; column < bmap->width; ++column) {
			for (plane = 0; plane < bmap->nplanes; plane++)
                tuplerow[column][plane] = roundsample(
                        bmap->m[plane]->data[row *
                            bmap->m[plane]->tda + column]);
		}
        pnm_writepamrow(&outpam, tuplerow);
	}

	/* Free the pnm memory */
	pnm_freepamrow(tuplerow);
	return 0;
}
