#include <gsl/gsl_matrix.h>

/*
 *	Y  =  0.29900 * R + 0.58700 * G + 0.11400 * B
 *	Cb = -0.16874 * R - 0.33126 * G + 0.50000 * B  + CENTERJSAMPLE
 *	Cr =  0.50000 * R - 0.41869 * G - 0.08131 * B  + CENTERJSAMPLE
 */

void
rgb2ycc(const gsl_matrix *m0, const gsl_matrix *m1, const gsl_matrix *m2)
{
	const double centersample=128;
	double r,g,b;
	double* row_element[3];
	int row, column;

	/* !!! the sizes must be the same for all the matrices */

	for (row = 0; row < m1->size1; row++)
	{
		row_element[0] = &m0->data[row * m0->tda];
		row_element[1] = &m1->data[row * m1->tda];
		row_element[2] = &m2->data[row * m2->tda];
		for (column = 0; column < m1->size2; column++)
		{
			r = *row_element[0];
			g = *row_element[1];
			b = *row_element[2];

			/* Assign Y, Cb, Cr */
			*row_element[0] = 0.29900 * r
				+ 0.58700 * g
				+ 0.11400 * b;
			*row_element[1] = -0.16874 * r
				- 0.33126 * g
				+ 0.50000 * b
				+ centersample;
			*row_element[2] = 0.50000 * r
				- 0.41869 * g
				- 0.08131 * b
				+ centersample;

			row_element[0]++;
			row_element[1]++;
			row_element[2]++;
		}
	}
}
