#define JPEG_INTERNALS
#include "jinclude.h"
#include "jpeglib.h"
#include "jdrest.h"

LOCAL(void) rest_decompress_data_normal
  JPP((j_decompress_ptr cinfo, int component,
	JSAMPARRAY output_ptr, JBLOCKARRAY blockarray));

LOCAL(int)
target2component(j_decompress_ptr cinfo, J_REST_TARGET target);

typedef struct jpeg_restoration * (*rest_adder)(j_decompress_ptr,
    J_REST_TARGET);

struct smethod_list
{
  char text[30];
  rest_adder adder;
};

static const struct smethod_list method_list[] = 
{
  { "nosratinia", jpeg_rest_nosratinia },
  { "froment", jpeg_rest_froment },
  { "orourke", jpeg_rest_orourke },
  { "hp", jpeg_rest_hp },
  { "robertson", jpeg_rest_robertson },
  { "", NULL }
};

LOCAL(void)
apply_plane_idct(j_decompress_ptr cinfo, J_REST_TARGET target,
    JSAMPIMAGE output_buf, const jvirt_barray_ptr whole_image[])
{
  struct jpeg_restoration *myrest;
  jpeg_component_info * compptr;
  JBLOCKARRAY block_buffer;
  boolean plane_applied = FALSE;
  int component;

  component = target2component(cinfo, target);

  /* Get the varray */
  compptr = &cinfo->comp_info[component];
  block_buffer = (*cinfo->mem->access_virt_barray)
    ((j_common_ptr) cinfo, whole_image[component],
     0, cinfo->total_iMCU_rows * compptr->v_samp_factor, FALSE);

  for(myrest = cinfo->rest; myrest != NULL; myrest = myrest->next)
  {
    if (myrest->target == target && plane_applied == FALSE)
    {
      /* Don't bother to IDCT an uninteresting component. */
#warning TODO: The "needness" must be updated depending on the methods used
      if (! compptr->component_needed)
	continue;

      myrest->decompress_plane(cinfo, myrest,
	  output_buf[component], block_buffer);

      plane_applied = TRUE;
    }
    else if (myrest->target == target && plane_applied == TRUE)
      ERREXIT(cinfo, JERR_REST_INVALID);
    else
      continue;
  }

  if (!plane_applied)
    rest_decompress_data_normal(cinfo, component,
	output_buf[component], block_buffer);
}

GLOBAL(void)
rest_process_restorators(j_decompress_ptr cinfo,
    JSAMPIMAGE output_buf, const jvirt_barray_ptr whole_image[])
{
#warning TODO: Check colorspace
  /* Process the plane unpacking */
  apply_plane_idct(cinfo, JREST_TARGET_IDCT_Y, output_buf, whole_image);
  if (cinfo->num_components > 1)
  {
    apply_plane_idct(cinfo, JREST_TARGET_IDCT_Cb, output_buf, whole_image);
    apply_plane_idct(cinfo, JREST_TARGET_IDCT_Cr, output_buf, whole_image);
  }
}

LOCAL(int)
target2component(j_decompress_ptr cinfo, J_REST_TARGET target)
{
  int component;

  switch(target)
  {
    case JREST_TARGET_IDCT_Y:
      component = 0;
      break;
    case JREST_TARGET_IDCT_Cb:
      component = 1;
      break;
    case JREST_TARGET_IDCT_Cr:
      component = 2;
      break;
    case JREST_TARGET_Y:
      component = 0;
      break;
    case JREST_TARGET_Cb:
      component = 1;
      break;
    case JREST_TARGET_Cr:
      component = 2;
      break;
    default:
      component = -1;
      ERREXIT(cinfo, JERR_REST_INVALID);
  }

  return component;
}

GLOBAL(void)
add_restorator(j_decompress_ptr cinfo, struct jpeg_restoration *the_rest)
{
  struct jpeg_restoration * myrest;

  the_rest->next = NULL;
  the_rest->component = target2component(cinfo, the_rest->target);

  /* Allocate for cinfo->rest */
  if (cinfo->rest == NULL) {
    cinfo->rest = the_rest;
  } else if (cinfo->rest != NULL) {
    /* Add to the restoration chain. */
    for(myrest = cinfo->rest; myrest->next != NULL; myrest = myrest->next)
      ;
    myrest->next = the_rest;
  } else
    ERREXIT(cinfo, JERR_NOT_COMPILED);
}

/*
 * Decompress and return some data in the multi-pass case.
 * Always attempts to emit one fully interleaved MCU row ("iMCU" row).
 * Return value is JPEG_ROW_COMPLETED, JPEG_SCAN_COMPLETED, or JPEG_SUSPENDED.
 *
 * NB: output_buf contains a plane for each component in image.
 */

LOCAL(void)
rest_decompress_data_normal (j_decompress_ptr cinfo, int component,
    JSAMPARRAY output_ptr, JBLOCKARRAY blockarray)
{
  JDIMENSION last_iMCU_row = cinfo->total_iMCU_rows - 1;
  JDIMENSION block_num;
  int block_row, block_rows;
  JBLOCKARRAY buffer;
  JBLOCKROW buffer_ptr;
  JDIMENSION output_col;
  jpeg_component_info *compptr;
  inverse_DCT_method_ptr inverse_DCT;

  compptr = &cinfo->comp_info[component];

  /* Align the virtual buffer for this component. */
  /*
  buffer = (*cinfo->mem->access_virt_barray)
    ((j_common_ptr) cinfo, coef->whole_image[ci],
     cinfo->output_iMCU_row * compptr->v_samp_factor,
     (JDIMENSION) compptr->v_samp_factor, FALSE);
     */
  buffer = &blockarray[cinfo->output_iMCU_row * compptr->v_samp_factor];
  
  /* Count non-dummy DCT block rows in this iMCU row. */
  if (cinfo->output_iMCU_row < last_iMCU_row)
    block_rows = compptr->v_samp_factor;
  else {
    /* NB: can't use last_row_height here; it is input-side-dependent! */
    block_rows = (int) (compptr->height_in_blocks % compptr->v_samp_factor);
    if (block_rows == 0) block_rows = compptr->v_samp_factor;
  }
  inverse_DCT = cinfo->idct->inverse_DCT[component];
  /* Loop over all DCT blocks to be processed. */
  for (block_row = 0; block_row < block_rows; block_row++) {
    buffer_ptr = buffer[block_row];
    output_col = 0;
    for (block_num = 0; block_num < compptr->width_in_blocks; block_num++) {
      (*inverse_DCT) (cinfo, compptr, (JCOEFPTR) buffer_ptr,
		      output_ptr, output_col);
      buffer_ptr++;
      output_col += compptr->DCT_scaled_size;
    }
    output_ptr += compptr->DCT_scaled_size;
  }
}

GLOBAL(RSAMPARRAY)
rest_rsarray_alloc(j_decompress_ptr cinfo, const int width, const int height)
{
	int brow;
	size_t memsize;
	char * offset; /* This should be one-byte datatype. is char sure? */
	RSAMPARRAY new;

	/* We should allocate all the size for the arrays at once. That is:
	 * sizeof(RBLOCKplane) * nplanes
	 * + sizeof(RBLOCKROW) * brows[bplane]
	 * + sizeof(RBLOCK) * bcolumns[bplane][brows]
	 */
	memsize=0;
	memsize+=sizeof(RSAMPROW) * height;
	memsize+=sizeof(RSAMP) * width * height;
	fprintf(stderr,"Allocating for %i bytes.\n", memsize);

	new = (RSAMPARRAY) (*cinfo->mem->alloc_large) ((j_common_ptr) cinfo,
	    JPOOL_IMAGE, memsize);
	if(new == NULL)
	{
	  ERREXIT1(cinfo, JERR_OUT_OF_MEMORY, 0);
	}

	/* Fill the structure */
	offset = (char *) new;
	/* Reserve for the RBLOCKplane rows */
	offset += sizeof(RSAMPROW) * height;
	for(brow = 0; brow < height; ++brow)
	{
		new[brow] = (RSAMPROW) offset;
		offset+=sizeof(RSAMP) * width;
	}
	printf("End offset: %i bytes.\n", offset - (char *) new);

	return new;
}

GLOBAL(RBLOCKARRAY)
rest_rbarray_malloc(j_decompress_ptr cinfo, const int width,
    const int height)
{
	int brow;
	size_t memsize;
	char * offset; /* This should be one-byte datatype. is char sure? */
	RBLOCKARRAY new;

	/* We should allocate all the size for the arrays at once. That is:
	 * sizeof(RBLOCKROW) * brows[bplane]
	 * + sizeof(RBLOCK) * bcolumns[bplane][brows]
	 */
	memsize=0;
	memsize+=sizeof(RBLOCKROW) * height;
	memsize+=sizeof(RBLOCK) * width * height;
	fprintf(stderr,"Allocating for %i bytes.\n", memsize);

	new = (RBLOCKARRAY) (*cinfo->mem->alloc_large) ((j_common_ptr) cinfo,
	    JPOOL_IMAGE, memsize);
	if(new == NULL)
	{
	  ERREXIT1(cinfo, JERR_OUT_OF_MEMORY, 0);
	}

	/* Fill the structure */
	offset = (char *) new;
	/* Reserve for the RBLOCKplane rows */
	offset += sizeof(RBLOCKROW) * height;
	for(brow = 0; brow < height; ++brow)
	{
		new[brow] = (RBLOCKROW) offset;
		offset+=sizeof(RBLOCK) * width;
	}
	printf("End offset: %i bytes.\n", offset - (char *) new);

	return new;
}

GLOBAL(RCOEFBUFFER)
rest_rcbuffer_malloc(j_decompress_ptr cinfo, const int elements)
{
  return (RCOEFBUFFER) (*cinfo->mem->alloc_large) ((j_common_ptr) cinfo,
      JPOOL_IMAGE, (size_t) elements*sizeof(RCOEF));
}

GLOBAL(RSAMPBUFFER)
rest_rsbuffer_malloc(j_decompress_ptr cinfo, const int elements)
{
  return (RSAMPBUFFER) (*cinfo->mem->alloc_large) ((j_common_ptr) cinfo,
      JPOOL_IMAGE, (size_t) elements*sizeof(RSAMP));
}

GLOBAL(void)
rest_rsbuffer_zero(RSAMPBUFFER buf, const int elements)
{
  memset((void *) buf, 0, elements*sizeof(RSAMP));
}

LOCAL(rest_adder)
find_adder(char *t)
{
  int p;

  p = 0;
  while(method_list[p].adder != NULL)
  {
    if(strcmp(t, method_list[p].text) == 0)
      return method_list[p].adder;

    p++;
  }

  return NULL;
}

LOCAL(void)
target_env(j_decompress_ptr cinfo, char *var, J_REST_TARGET plane)
{
  char *val;
  rest_adder a;

#ifndef NO_GETENV
  val = getenv(var);
  if (val != NULL)
  {
    a = find_adder(val);
    if (a != NULL)
      /* Run the 'adder' function, if found */
      a(cinfo, plane);
  }
#endif
}

GLOBAL(void)
rest_env(j_decompress_ptr cinfo)
{
  target_env(cinfo, "JPEG_REST_TARGET_IDCT_Y", JREST_TARGET_IDCT_Y);
  target_env(cinfo, "JPEG_REST_TARGET_IDCT_Cb", JREST_TARGET_IDCT_Cb);
  target_env(cinfo, "JPEG_REST_TARGET_IDCT_Cr", JREST_TARGET_IDCT_Cr);
  target_env(cinfo, "JPEG_REST_TARGET_IDCT_YCrCb", JREST_TARGET_IDCT_YCrCb);
  target_env(cinfo, "JPEG_REST_TARGET_Y", JREST_TARGET_Y);
  target_env(cinfo, "JPEG_REST_TARGET_Cb", JREST_TARGET_Cb);
  target_env(cinfo, "JPEG_REST_TARGET_Cr", JREST_TARGET_Cr);
  target_env(cinfo, "JPEG_REST_TARGET_YCrCb", JREST_TARGET_YCrCb);
  target_env(cinfo, "JPEG_REST_TARGET_R", JREST_TARGET_R);
  target_env(cinfo, "JPEG_REST_TARGET_G", JREST_TARGET_G);
  target_env(cinfo, "JPEG_REST_TARGET_B", JREST_TARGET_B);
  target_env(cinfo, "JPEG_REST_TARGET_RGB", JREST_TARGET_RGB);
}
